<?php

/**
 * Save Account to database 
 */
function gbuzz_account_save($account) {
	$values = (array)$account;

  if (db_result(db_query("SELECT 1 FROM {gbuzz_account} WHERE id = %n", $values['id']))) {
    drupal_write_record('gbuzz_account', $values, array('id'));
  }
  else {
    drupal_write_record('gbuzz_account', $values);
  }    
}

/**
 * Load Buzz user account
 * 
 * @params $params
 *    Either the google profile id  or an array of conditions to match against account in database
 * @params $reset
 *    Whether to reset the internal account cache
 * 
 * @return 
 *    A GBuzzUser object if google id is passed, or an array of GBuzzUser objects matching the conditions
 */
function gbuzz_account_load($params, $reset = FALSE) {
  module_load_include('lib.php', 'gbuzz');
	static $accounts = array();
  
  if ($reset) {
  	$accounts = array();
  }
    
  if (is_numeric($params)) {
  	if(isset($accounts[$params])) {
  		return $accounts[$params];
  	}
    $cond = 'id = %n';
    $args[] = $params;
  }
  else if(is_array($params)) {
  	foreach ($params as $key => $value) {
      if ($key == 'id') {
      	$cond[] = db_escape_table($key) . "= %n";
        $args[] = $value;
      }
      else if(in_array($key, array('uid', 'is_global', 'import'))) {
      	$cond[] = db_escape_table($key) . "= %d";
        $args[] = $value;
      }
      else {
  		  $cond[] = db_escape_table($key) . "= '%s'";
        $args[] = $value;
      }
  	}
    $cond = implode(' AND ', $cond);
  }
  else {
  	return FALSE;
  }
  $query = db_query("SELECT * FROM gbuzz_account WHERE ". $cond, $args);
  while ($gbuzz_account = db_fetch_array($query)) {
    $account[$gbuzz_account['id']] = $gbuzz_accounts[$gbuzz_account['id']] = new GBuzzUser($gbuzz_account);
  }

  return is_numeric($params)? $gbuzz_accounts[$params]: $gbuzz_accounts;
}

/**
 * Delete a Buzz user from database
 * 
 * @params $google_id
 *    Google Id associated with the account
 */

function gbuzz_user_delete($google_id) {
	$sql = "DELETE FROM {gbuzz_account} WHERE id = %n";
  db_query($sql, $google_id);
}

/**
 * Create an Activity
 * 
 * @see http://code.google.com/apis/buzz/v1/using_rest.html#create-activity
 */ 
function gbuzz_activity_create(&$activity, $content = '') {
  $activity['data']['object'] = array(
    'type' => 'note',
    'content' => $content,
  );
}

/**
 * Create an attachment of type 'article' associated with the activity
 * 
 * @see http://code.google.com/apis/buzz/v1/using_rest.html#article-attachment
 */ 
function gbuzz_activity_attach_article(&$activity, $title, $node_url) { 
    
  $activity['data']['object']['attachments'][] = array(
    'type' => 'article',
    'title' => $title,
    'links' => array(
      'alternate' => array(
        array(
          'href' => $node_url,
        ),
      ),
    ),
  ); 
}

/**
 * Attach an image to an activity using imagefield module (filename should not have space)
 * 
 * @see http://code.google.com/apis/buzz/v1/using_rest.html#photo-attachment
 */ 
function gbuzz_activity_attach_photo(&$activity, $field, $node_url) {
  global $base_url;   
  
  foreach($field as $field_value) {
    $filepath = $base_url . '/' . $field_value['filepath'];
    $filemime = $field_value['filemime'];
    
    if (!$field_value['status']) {
    	continue;
    }   
    // Value to be used as description for an image
    // Priority description <- alt
    if ($field_value['data']['description'] || $field_value['data']['alt']) {
      if ($field_value['data']['description']) {
        $description = $field_value['data']['description'];
      }
      else {
        $description = $field_value['data']['alt'];
      }
    }
    $title = $field_value['data']['title'];
    
    $activity['data']['object']['attachments'][] = array(
      'type' => 'photo',
      'content' => $description,
      'title' => $title,
      'links' => array(
        'enclosure' => array(array('href' =>  $filepath, 'type' => $filemime)),
        'preview' => array(array('href' => $filepath, 'type' => $filemime)),
        'alternate' => array(array('href' => $node_url,'type' => 'text/html')),
      ),
    );
  
  }
}

/**
 * Attach 3rd party images with an activity
 */
function gbuzz_activity_attach_emimage(&$activity, $field) {

  foreach($field as $field_value) {
    
    $image_url = $field_value['embed'];
    $thumbnail_url = $field_value['data']['thumbnail']['url'];
    $provider = $field_value['provider'];
    
    if (!$image_url) {
    	continue;
    }
    
    $activity['data']['object']['attachments'][] = array(
      'type' => 'photo',
      'links' => array(
        'preview' => array(array('href' => $thumbnail_url, 'type' => 'image/jpeg')),
        'alternate' => array(array('href' => $image_url, 'type' => 'text/html')),
      ),
    );
    
  }
}

/**
 * Attach a 3rd party video with an activity
 * 
 * @see http://code.google.com/apis/buzz/v1/using_rest.html#video-attachment
 */
function gbuzz_activity_attach_emvideo(&$activity, $field) {

  foreach($field as $field_value) {
    $video_url = $field_value['embed'];
    $thumbnail_url = $field_value['data']['thumbnail']['url'];
    $provider = $field_value['provider'];

    // Buzz has video support for Youtube video attachments 
    if (!$video_url || $provider != 'youtube') {
      continue;
    }
    $flash_url = $field_value['data']['flash']['url'];
    $flash_mime = $field_value['data']['flash']['mime'];
    $activity['data']['object']['attachments'][] = array(
      'type' => 'video',
      'links' => array(
        'preview' => array(array('href' => $thumbnail_url, 'type' => 'image/jpeg')),
        'alternate' => array(array('href' => $video_url, 'type' => 'text/html'), array('href' => $flash_url, '$type' => $flash_mime)),
      ),
    );
    
  }
}

function gbuzz_make_storage_fields($case) {
  return preg_replace('/(.*?[a-z]{1})([A-Z]{1}.*?)/', '${1}_${2}', $case);
}

/**
 * Helper function, fetches Account details and pass it to gbuzz_account_save to save in database
 */
function _gbuzz_user_make($values) {
  $account = array();    
  if(!isset($values['uid'])) {
    global $user;
    $values['uid'] = $user->uid;
  }

  $account['id']                 = $values['id'];
  $account['uid']                = $values['uid'];
  $account['kind']               = $values['kind'];
  $account['about']              = $values['aboutMe'];
  $account['display_name']       = $values['displayName'];
  $account['profile_url']        = $values['profileUrl'];
  $account['thumbnail_url']      = $values['thumbnailUrl'];
  $account['emails']             = serialize($values['emails']);
  $account['urls']               = serialize($values['urls']);
  $account['photos']             = serialize($values['photos']);
  $account['organizations']      = serialize($values['organizations']);
  $account['interests']          = serialize($values['interests']);
  $account['oauth_token']        = $values['oauth_token'];
  $account['oauth_token_secret'] = $values['oauth_token_secret'];
  $account['import']             = isset($values['import'])? $values['import']: 1;
  
  return $account;
}